<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category template manager
 * @author Ruben Waitz
 * @copyright Copyright (c) 2022 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since June 10, 2022
 * @version $Rev$
 */

/**
 * Admin_Settings class
 * Used to enable/disable bleeding edge features
 */
class Admin_Diagnostics extends WP_Page {

    protected $interfacer = null;

	/** @noinspection PhpMissingParentConstructorInspection */
	public function __construct() {
        $this->interfacer = WP_Interfacer::getInstance();
    }

    public function getCapability() {
        return WP_Constants::CAPABILITY_ADMIN;
    }

    public function getMenuSlug() {
        return 'carerix_diagnostics_page';
    }

    public function getMenuTitle() {
        return 'Diagnostics <span style="color:red;">(new)</span>';
    }

    public function getPageTitle() {
        return 'Diagnostics';
    }

	public function page() {
        $template = new Utils_Template('admin/forms/diagnostics'); // wp-content/plugins/CxWordpress/templates/admin/forms/diagnostics.php
        $template->with()->display();
    }

	private function cx_API_connect()
	{
		$app_name = Environment_Installation::getAppName();
		$app_token = Environment_Installation::getAppToken();
		$client = new Carerix\Api\Rest\Client();
		$manager = new Carerix\Api\Rest\Manager($client);
		$manager->autoDiscoverEntities();
		$manager->setUrl(WP_Constants::CARERIX_API_ENDPOINT);
		$manager->setUsername($app_name);
		$manager->setPassword($app_token);

		Carerix\Api\Rest\Entity::setManager($manager);
	}

	/**
	 * Checks the Carerix API connection
	 *
	 * @return string|null  null=OK, else a string in case of an error
	 */
	public function getCxApiConnectionStatus() : ?string
	{
		try {
			$this->cx_API_connect();
			$params = array(
				'start' => '0',
				'count' => '1',
				'qualifier' => "isMainAgency = 1",
				'show' => array("agencyID", "name", "shortName"));
			/*$data_raw = */Carerix\Api\Rest\Entity\CRAgency::findAll($params);
			// $xx = $data_raw->getArrayCopy();
			// print_r($xx);
			return null;
		} catch (Exception $care) { // The \Carerix\Api\Rest\Exception handler doesn't work, so we do it ourselves
			$value = $care->getMessage();
			try {
				$sxe = @ new SimpleXMLElement($value);
			} catch (\Exception $ex) {
				return "Unable to parse exception error";
			}

			$message = (string)$sxe->reason;
			return $message;
		}
	}

	public function getDiagnosticsGeneral() : array {
		$app_name = Environment_Installation::getAppName();
		$app_token = Environment_Installation::getAppToken();

		$app_token_length = strlen($app_token);
		$app_token_hashed = substr($app_token, 0, 5) . str_repeat(".", $app_token_length-5-5) . substr($app_token, -5, 5);

		$memory_limit_raw = ini_get('memory_limit');
		$memory_limit_int = wp_convert_hr_to_bytes($memory_limit_raw);

		$memory_limit_mb  = -1; // -1 is unlimited
		if ($memory_limit_int !== -1) {
			$memory_limit_mb = $memory_limit_int / MB_IN_BYTES;
		}

		$cx_api_systemstatus = $this->getCxApiConnectionStatus();

		if ( !isset($cx_api_systemstatus) ) {
			$status_msg = "OK";
		} else {
			$status_msg = "<b style='color:red;'>{$cx_api_systemstatus}</b>";
		}


		$return_values = [];

		$return_values[] = ['item' => "App name", 'value' => $app_name];
		$return_values[] = ['item' => "Rest API token", 'value' => $app_token_hashed];
		$return_values[] = ['item' => "Connection to Carerix REST API server", 'value' => $status_msg];
		$return_values[] = ['item' => "Carerix WordPress Plugin version", 'value' => Carerix::VERSION];
		$return_values[] = ['item' => "WordPress version", 'value' => get_bloginfo('version')];
		$return_values[] = ['item' => "PHP version", 'value' => PHP_VERSION];
		$return_values[] = ['item' => "PHP memory limit", 'value' => ($memory_limit_mb == -1 ? "unlimited" : $memory_limit_mb . " MB")];

		return $return_values;
	}

	public function getDiagnosticsMediumChannels( $asRawData = false ) : array {
		$this->cx_API_connect();

		// try to fetch one medium to check if authentication works
		$params = array(
			'start'    => '0',
			'count'    => '200',
			'ordering' => "({key=code;sel=Ascending})"
			/*'qualifier' => "code like 'web*'",*/ // where SQL statement
		);


		try {
			$data = Carerix\Api\Rest\Entity\CRMedium::findAll($params);
			$arrMedium = $data->getArrayCopy();
		}
		catch (Exception $e) {
			$arrMedium = [];
		}

		if ($asRawData) {
			return $arrMedium;
		}

		$return_values = [];
		$medium_idx = 0;
		foreach ($arrMedium as $cr_medium)
		{
			$code                  = $cr_medium->getCode() ?? "";
			$name                  = $cr_medium->getName() ?? "";
			$isHtml                = (int) $cr_medium->isHTML ? "Formatting activated" : "Only plain text enabled";
			$isDefaultMedium       = ($code == "web");
			$isMultilanguageMedium = (substr($code, 0, 4) == "web-");
			$languageExtension     = substr($code, 4);
			$notes                 = "";
			if ($isDefaultMedium) $notes = "This is the default medium that this plugin uses to fetch the vacancies.";
			if ($isMultilanguageMedium) $notes = "This is a medium for the language-code <span style='font-weight: bold; background-color: yellow; border: rgb(128,128,128) solid 1px; padding: 3px;'>{$languageExtension}</span>. It's used for multilingual sites. You need to install and configure the third-party WPML-plugin to make it work.";
			if ((int) $cr_medium->isHTML == 0) $notes .= " Rich formatting in the Carerix editor is disabled. (Ask your Carerix system administrator to set it up)";
			if (strlen(trim($code)) == 0) $notes = "You cannot use this one because it misses a medium-code. (If you are planning to use this, ask your Carerix system administrator to set it up)";
			$notes = trim($notes);

			/* <?php echo strlen($isDefaultMedium) ? "font-weight: bold; ": ""; ?>background-color: <?php echo strlen($code) ? "#00ab6b": "#cc0000"; ?>; color: <?php echo strlen($code) ? "#000000": "#ffffff"; ?>" */

			$textFormatting = [];
			if (strlen($isDefaultMedium)) $textFormatting[] = "font-weight: bold;";

			$styleOpen = "<span style=\"" . implode(" ", $textFormatting) . "\">";
			$styleClose = "</span>";

			$rowCssClasses = [];
			$rowCssClasses[] = strlen($code) ? ($medium_idx % 2 == 0 ? "carerix_diagnostics_row_ok" : "carerix_diagnostics_row_ok2") : ""; // "carerix_diagnostics_row_nok";
			$row_css = implode(" ", $rowCssClasses);

			$row = [];
			$row['code'] = "{$styleOpen}{$code}{$styleClose}";
			$row['name'] = "{$styleOpen}{$name}{$styleClose}";
			$row['ishtml'] = "{$styleOpen}{$isHtml}{$styleClose}";
			$row['notes'] = "{$styleOpen}{$notes}{$styleClose}";
			$row['rowcss'] = $row_css;
			$return_values[] = $row;

			$medium_idx++;
		}

		return $return_values;
	}

	public function getDiagnosticsActivePublications( array $request, int $start, int $length, string $keyword = "" ) : array {
		$columns = array(
			array( 'db' => 'publicationID', 'dt' => 'publicationID' ),
			array( 'db' => 'titleInformation', 'dt' => 'titleInformation' ),
		);
		$ordering = self::order( $request, $columns );

		$this->cx_API_connect();

		$data = [];
		$recordsTotal = 0;

/*		$params = array(
			'start'    => $start,
			'count'    => $length,
			'medium' => "web",
			'keyword' => $keyword,
			'show' => array("publicationID", "titleInformation", "toVacancy.vacancyID", "toVacancy.jobTitle"),
		);
		if ( !empty($ordering )) $params['ordering'] = $ordering; // // "({key=publicationID;sel=Ascending})"

		try {
			$cx_data = Carerix\Api\Rest\Entity\CRPublication::listBy($params);
			$recordsTotal = (int) $cx_data->getTotalCount();
			$arrData = $cx_data->toArray(); // getArrayCopy();
		}
		catch (Exception $e) {
			$arrData = [];
		}*/


		$medium_codes = $this->interfacer->getMediumCodesFromRegisteredSources();
		$medium_codes = implode("|", $medium_codes);

		$qualifier = $this->interfacer->getPublicationsListQualifier($medium_codes);
		$qualifier_IDs = "";
		if ( is_numeric($keyword) ) {
			$id = (int) $keyword;
			$qualifier_IDs = "OR toVacancy.vacancyID={$id} OR publicationID={$id}";
		}
		$qualifier .= " and ( toVacancy.jobTitle caseInsensitiveLike '*{$keyword}*' OR titleInformation caseInsensitiveLike '*{$keyword}*' {$qualifier_IDs})";
		$params = array(
			'start'    => $start,
			'count'    => $length,
			'qualifier' => $qualifier,
			'show' => array("publicationID", "titleInformation", "toVacancy.vacancyID", "toVacancy.jobTitle", "toMedium.code", "toVacancy.isAnonymous", "toVacancy.isSalaryPublic"),
		);
		if ( !empty($ordering )) $params['ordering'] = $ordering; // // "({key=publicationID;sel=Ascending})"

		try {
			$cx_data = Carerix\Api\Rest\Entity\CRPublication::findAll($params);
			$recordsTotal = (int) $cx_data->getTotalCount();
			$arrData = $cx_data->toArray(); // getArrayCopy();
		}
		catch (Exception $e) {
			$arrData = [];
		}


//		$recordsFiltered = count($arrData); // unused?

		foreach ($arrData as $item) {
			$row = [];
			$row['publicationID'] = (int) $item['publicationID'];
			$row['titleInformation'] = $item['titleInformation'];
			$row['vacancyID'] = (int) $item['toVacancy']['vacancyID'];
			$row['jobTitle'] = $item['toVacancy']['jobTitle'];
			$row['mediumCode'] = $item['toMedium']['code'];
			$row['isAnonymous'] = (int) ($item['toVacancy']['isAnonymous'] ?? 0); // Show/Hide company name
			$row['isSalaryPublic'] = (int) ($item['toVacancy']['isSalaryPublic'] ?? 0);
			$data[] = $row;
		}

		$output = array();
		$output['recordsTotal'] = $recordsTotal;
		$output['recordsFiltered'] = $recordsTotal; // $recordsFiltered;
		$output['data'] = $data;

		return $output;
	}

	/**
	 * Ordering
	 *
	 * Construct the ORDER BY clause for server-side processing SQL query
	 *
	 *  @param  array $request Data sent to server by DataTables
	 *  @param  array $columns Column information array
	 *  @return string SQL order by clause
	 */
	static function order ( $request, $columns )
	{
		$order = '';

		if ( isset($request['order']) && count($request['order']) ) {
			$orderBy = array();
			$dtColumns = self::pluck( $columns, 'dt' );

			for ( $i=0, $ien=count($request['order']) ; $i<$ien ; $i++ ) {
				// Convert the column index into the column data property
				$columnIdx = intval($request['order'][$i]['column']);
				$requestColumn = $request['columns'][$columnIdx];

				$columnIdx = array_search( $requestColumn['data'], $dtColumns );
				$column = $columns[ $columnIdx ];

				if ( $requestColumn['orderable'] == 'true' ) {
					$dir = $request['order'][$i]['dir'] === 'asc' ?
						'Ascending' :
						'Descending';

					$orderBy[] = '{key='.$column['db'].';sel='.$dir.'}';
				}
			}

			if ( count( $orderBy ) ) {
				$order = '('.implode(', ', $orderBy) . ')';
			}
		}

		return $order;
	}

	/**
	 * Pull a particular property from each assoc. array in a numeric array,
	 * returning and array of the property values from each item.
	 *
	 *  @param  array  $a    Array to get data from
	 *  @param  string $prop Property to read
	 *  @return array        Array of property values
	 */
	static function pluck ( $a, $prop )
	{
		$out = array();

		for ( $i=0, $len=count($a) ; $i<$len ; $i++ ) {
			if(empty($a[$i][$prop])){
				continue;
			}
			//removing the $out array index confuses the filter method in doing proper binding,
			//adding it ensures that the array data are mapped correctly
			$out[$i] = $a[$i][$prop];
		}

		return $out;
	}



} // class