<?php

class Admin_ImportExport extends WP_Page {

    protected $interfacer = null;

    public function __construct() {
    	return; // disable
        $this->interfacer = WP_Interfacer::getInstance();
    }

    public function getCapability() {
        return WP_Constants::CAPABILITY_ADMIN;
    }

    public function getMenuSlug() {
        return 'carerix_importexport_page';
    }

    public function getMenuTitle() {
        return 'Import / Export';
    }

    public function getPageTitle() {
        return 'Import / Export';
    }

    public function page() {
        // sections 
        //- general settings
        //- forms
        //- forms settings
        //- sources
        //-- jobs
        //-- promotional jobs
        //-- candidates
        //-- people
        //-- companies
        //-- newsletters
        //- taxonomies

        $data = array();
        $input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

        if (isset($input_data['export_to_file']) || isset($input_data['export_to_textarea'])) {

            $d = $this->export($input_data);
            $export = serialize($d);
            if (isset($input_data['export_to_textarea'])) {
                $data['export'] = $export;
            }

            if (isset($input_data['export_to_file'])) {
                $uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . 'exports';
                if (!file_exists($uploadDirPath))
                    mkdir($uploadDirPath);

                $file = tempnam($uploadDirPath, 'export');
                file_put_contents($file, $export);

                $url = plugins_url() . '/' . basename(dirname(dirname(__FILE__))) . '/download.php?q=' . basename($file);
                echo "<iframe src='$url' style='display: none;'></iframe>";
            }
        }

        if (isset($input_data['import_from_file']) || isset($input_data['import_from_text'])) {
            if (isset($input_data['import_from_text']) && strlen(trim($input_data['import_text'])) > 0) {
                $d = stripslashes($input_data['import_text']);
                $doc = unserialize($d);
            }

            if (isset($input_data['import_from_file'])) {
                if (isset($_FILES['import_from_file']) && $_FILES['import_from_file']['error'] === 0) {
                    $path = $_FILES['import_from_file']['tmp_name'];
                    $data = file_get_contents($path);
                    $doc = unserialize($data);
                }
            }

            $this->import($doc);
        }

        $template = new Utils_Template('admin/importexport');
        $template->with($data)->display();
    }

    public function export($data) {
        $export = array();
        if (isset($data['general'])) {
            $export['general'] = $this->export_settings();
        }

        if (isset($data['forms'])) {
            $export['forms'] = $this->export_forms();
        }

        if (isset($data['sources'])) {
            $export['sources'] = $this->export_sources();
        }

        if (isset($data['taxonomies'])) {
            $export['taxonomies'] = $this->export_taxonomies();
        }

        return $export;
    }

    private function export_settings() {
        // appname, apptoken, report
        return array(
            'app_name' => Environment_Settings::getAppName(),
            'app_token' => Environment_Settings::getAppToken(),
            'app_reporting' => Environment_Settings::getAppReporting()
        );
    }

    private function export_forms() {
        // forms + form settings (login link, extra apply, birthdate, agreement_label, agreement_url, agreement_text, captcha, 
        $formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
        $applyForms = $formsTable->getByType(WP_Constants::ENTITY_TYPE_FORM_APPLY);

        return array(
            'forms' => $applyForms,
            'form_settings' => array(
                Environment_Settings::KEY_FORM_AGREEMENT => Environment_Settings::getFormGeneralAgreement(),
                Environment_Settings::KEY_FORM_AGREEMENT_LABEL => Environment_Settings::getFormGeneralAgreementLabel(),
                Environment_Settings::KEY_FORM_AGREEMENT_URL => Environment_Settings::getFormGeneralAgreementUrl(),
                Environment_Settings::KEY_FORM_ADRESS_FORMAT => Environment_Settings::getFormAddressFormatGeneral(),
                Environment_Settings::KEY_APPLY_OPTIONS_STATUS => Environment_Settings::getFormApplyOptionsStatus(),
                Environment_Settings::KEY_APPLY_LOGIN_STATUS => Environment_Settings::getLoginLinkStatus(),
                Environment_Settings::KEY_APPLY_FORM_CAPTCHA => Environment_Settings::getApplyFormCaptchaStatus(),
                Environment_Settings::KEY_TAF_CAPTCHA => Environment_Settings::getTAFCaptchaStatus(),
                Environment_Settings::KEY_AGE_LIMIT => Environment_Settings::getAgeLimit(),
        ));
    }

    private function export_sources() {
        // jobs, promotional, candidates, people, companies, newsletters
        $sourcesTable = new WP_Table(Environment_Installation::TABLE_SOURCES);
        $sources = $sourcesTable->getAll();
        return array('sources' => $sources);
    }

    private function export_taxonomies() {
        // everything that starts like resource_taxonomy
        $taxonomiesTable = Environment_Installation::get_table_name(Environment_Installation::TABLE_SETTINGS);
        $query = "SELECT * FROM $taxonomiesTable WHERE `key` LIKE 'resource_taxonomy%'";
        $data = $this->interfacer->db_get_custom($query);
        return array('taxonomies' => $data);
    }

    public function import($data) {
        if (isset($data['general'])) {
            $general = $data['general'];
            if (isset($general['app_name']))
                Environment_Settings::setAppName($general['app_name']);
            if (isset($general['app_token']))
                Environment_Settings::setAppToken($general['app_token']);
            if (isset($general['app_reporting']))
                Environment_Settings::setAppReporting($general['app_reporting']);
        }

        if (isset($data['forms'])) {
            $forms = $data['forms']['forms'];
            $settings = $data['forms']['form_settings'];

            $formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
            foreach ($forms as $form) {
                $d = unserialize($form['settings']);
                if (isset($d['is_default']) && $d['is_default'] == 'on')
                    continue;

                unset($form['id']);
                $formsTable->insert($form);
            }

            foreach ($settings as $key => $value) {
                Environment_Settings::_set_setting_unique($key, $value);
            }
        }

        if (isset($data['sources'])) {
            $sourcesTable = new WP_Table(Environment_Installation::TABLE_SOURCES);
            $sources = $data['sources']['sources'];
            foreach ($sources as $source) {
                unset($source['id']);
                $sourcesTable->insert($source);
            }
        }

        if (isset($data['taxonomies'])) {
            $taxonomies = $data['taxonomies']['taxonomies'];
            foreach ($taxonomies as $taxonomy) {
                Environment_Settings::_set_setting_unique($taxonomy['key'], $taxonomy['value']);
            }
        }
    }
}
