<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category template manager
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 26, 2013 11:36:11 AM
 * @version $Rev$
 */

/**
 * Settings class
 * Used to ....
 */
class Admin_Sources_People extends WP_Page {

    protected $interfacer = null;
    private $source_type = 'people';

    public function __construct() {
        $this->interfacer = WP_Interfacer::getInstance();
    }

    public function getCapability() {
        return WP_Constants::CAPABILITY_ADMIN;
    }

    public function getMenuSlug() {
        return 'carerix_sources_people_page';
    }

    public function getMenuTitle() {
        return 'People';
    }

    public function getPageTitle() {
        return 'People';
    }

    public function page() {
        $input_data = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
        switch ($input_data['action']) {
            case 'edit':
                $this->sourceScreen($input_data['id']);
                break;
            case 'add':
                $this->sourceScreen();
                break;
            case 'delete':
                $this->deleteSource($input_data['id']);
                break;
            case 'association':
                $this->deleteAssociatedPosts($input_data['id']);
            case 'list':
            default:
                $this->listSources($input_data['message']);
                break;
        }
    }

    private function sourceScreen($id = false) {
        $input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
        $data = array();
        if ($input_data != NULL && wp_verify_nonce($input_data['_carerix_nonce_field'], 'source_edit')) {
            if (trim($input_data['name']) != '') {
                $input_data['introtext'] = $_POST['introtext'];
                unset($input_data['save']);
                $backup_data = $input_data;
                $id = false;
                if (isset($input_data['id']))
                    $id = $input_data['id'];

                $peopleSource = new Entity_Source_People($id);
                $peopleSource->save($input_data);

                $id = $peopleSource->getID();
                $data['edit_data'] = $backup_data;
            }
            else {
                $data['page_error'] = true;
                $data['title_error'] = true;
            }
        }

        if ($id) {
            $peopleSource = new Entity_Source_People($id);
            $sourceData = $peopleSource->getData();
            $sourceData['settings'] = unserialize($sourceData['settings']);

            if ($input_data !== null && !isset($data['page_error'])) {
                // $cronRunner = new Environment_CronRunner();
                // $cronRunner->runPeopleCronForSource($sourceData);
            }

            $data['edit_data'] = $sourceData;
        }
        if (!isset($data['page_error']) && $input_data != null) {
            $data['success'] = true;
        }
        $data['cancel_link'] = $this->createActionLink('list');
        $template = new Utils_Template('admin/sources/people_edit');
        $template->with($data)->display();
    }

    static public function ajaxSaveSource($input_data) {
        $data = array();
        if (trim($input_data['name']) != '') {
            //$input_data['introtext'] = $_POST['introtext'];
            $backup_data = $input_data;
            $id = false;
            if (isset($input_data['id']))
                $id = $input_data['id'];

            $peopleSource = new Entity_Source_People($id);
            $peopleSource->save($input_data);

            $id = $peopleSource->getID();
        }
        else {
            $data['page_error'] = true;
            $data['title_error'] = true;
        }

        if ($id) {
            $peopleSource = new Entity_Source_People($id);
            $sourceData = $peopleSource->getData();
            $sourceData['settings'] = unserialize($sourceData['settings']);

            if ($input_data !== null && !isset($data['page_error'])) {
//                $cronRunner = new Environment_CronRunner();
//                $cronRunner->runPeopleCronForSource($sourceData);
            }
            $data['id'] = $id;
        }

        if (!isset($data['page_error']) && $input_data != null) {
            $data['success'] = true;
        }
        return $data;
    }

    private function deleteSource($id) {
        $peopleSource = new Entity_Source_People($id);
        $peopleSource->delete();
        $data = array();
        $data['source_id'] = $id;
        $data['delete_link'] = $this->getSourceAssociationDeleteLink($id);
        $data['tab_link'] = $this->getTabLink();

        $template = new Utils_Template('admin/sources/delete_associations');
        $template->with($data)->display();
    }

    private function deleteAssociatedPosts($sourceID) {
        $entities = $this->interfacer->entities_get_where_property('source', $sourceID);
        foreach ($entities as $entity) {
            $this->interfacer->entity_delete($entity->post_id);
        }
    }

    private function getTabLink() {
        return 'admin.php?page=' . $this->getParentTab()->getMenuSlug() .
                '&tab=' . $this->getMenuSlug();
    }

    private function getSourceAssociationDeleteLink($source_id) {
        return $this->createActionLink('association') . '&id=' . $source_id;
    }

    private function listSources($message = '') {
        $data = array();

        $sourcesTable = new WP_Table(Environment_Installation::TABLE_SOURCES);

        $peopleSources = $sourcesTable->getByType($this->source_type);

        foreach ($peopleSources as &$peopleSource) {
            $peopleSource['settings'] = unserialize($peopleSource['settings']);
            $peopleSource['editLink'] = $this->getSourceEditLink($peopleSource['id']);
            $peopleSource['deleteLink'] = $this->getSourceDeleteLink($peopleSource['id']);
            $peopleSource['pageLink'] = $this->getFrontendPage($peopleSource['id']);
        }

        $data['add_new'] = $this->createActionLink('add');
        $data['peopleSources'] = $peopleSources;

        switch ($message) {
            case 'yes':
                $temp = new Utils_Template('admin/sources/confirmation');
                $temp->with(array('type' => 'yes'))->display();
                break;
            case 'no':
                $temp = new Utils_Template('admin/sources/confirmation');
                $temp->with(array('type' => 'no'))->display();
                break;
            default:
                break;
        }

        $template = new Utils_Template('admin/sources/people');
        $template->with($data)->display();
    }

    private function getFrontendPage($source_id) {
        $ents = $this->interfacer->entities_get_where_property('_carerix_people', $source_id);
        return $this->interfacer->get_post_permalink($ents[0]->post_id);
    }

    private function createActionLink($action_name) {
        return 'admin.php?page=' . $this->getParentTab()->getMenuSlug() .
                '&tab=' . $this->getMenuSlug() .
                '&action=' . $action_name;
    }

    private function getSourceEditLink($source_id) {
        return $this->createActionLink('edit') . '&id=' . $source_id;
    }

    private function getSourceDeleteLink($source_id) {
        return $this->createActionLink('delete') . '&id=' . $source_id;
    }

}
