<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 29, 2013 12:55:30 PM
 * @version $Rev$
 */


/**
 * Cron class
 * Used to define cron objects
 */
abstract class Entity_Cron {
//	const INIT_SYNC_STARTDATE = "1990-01-01 00:00:00";

	/**
	 * @var int $page Page offset
	 */
	protected $page = 0;


    /**
	 *
	 * @var WP_Interfacer 
	 */
	protected $interfacer = null;
	
	/**
	 *
	 * @var string 
	 */
	protected $url = '';
	
	/**
	 *
	 * @var array 
	 */
	protected $sourceData = array();
	
	/**
	 *
	 * @var boolean 
	 */
	protected $usesUrl = true;
	
	protected $afterFinishData = array();

	protected $manual_called = false;

	/**
	 *
	 * @param array $sourceData
	 * @param bool  $manual_called  "true" if the is sync called as "https://www.site.com/?carerix_sync"
	 */
	public function __construct($sourceData, $manual_called = false) {
		$this->interfacer = WP_Interfacer::getInstance();
		$this->sourceData = $sourceData;
		$this->manual_called = $manual_called;
  	}
	
	abstract function getUrl();
	abstract function process($data);
	
    
    public function setPage($page) {
        $this->page = $page;
    }
    
    public function getPage()
    {
        return $this->page;
    }

	/**
	 * Retrieves the CX data (jobs, people, etc.) by the cxtools/wp feeds
	 *
	 * @deprecated In the future must be phased out and refactored to an abstract method in favour to REST-API
	 * @see All entities in wp-content/plugins/CxWordpress/Entity/Cron/* will implement their own REST-API variant
	 * @return SimpleXML
	 * @throws Exception
	 */
	protected function retrieveData()
	{
		if(!$this->usesUrl) return;
		$page_offset = $this->page;
        $start = (int)$page_offset * WP_Constants::NR_OF_ITEMS;
        $url = $this->getUrl();
        $glue = '?';
        $token = Environment_Settings::getAppToken();
        if(stripos($url, '?') !== FALSE) $glue = '&';
        $newUrl = $url . $glue . 'start=' . $start.'&count='.WP_Constants::NR_OF_ITEMS.'&token='.$token;
        $ch = curl_init( $newUrl );
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $feed_string = curl_exec($ch);
        $response = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if($response == 200)
        {
            $feedData = simplexml_load_string($feed_string);
            return $feedData;
        } else
        {
            throw new Exception('Error: the synchronization process failed. Please try again. If the problem persists please contact your Carerix administrator. '
                    . "<br> Got response code $response while accessing $newUrl");
        }
	}

	
	/**
	 * 
	 * @return boolean
	 */
	private function urlIsValid()
	{
        return;
		$service_handle = fopen($this->getUrl(), 'r');
		if($service_handle === false) {
            return false;
        }
		return true;
	}
	
	/**
	 * sends the retrieved data to the process method of the cron mechanism
     * retrieves number of items processed
	 */
	public function run()
	{
		$data = $this->retrieveData();
		return $this->process($data);
		$data = null;
		unset($data);
	}
	
	public function getAfterFinish()
	{
		return $this->afterFinishData;
	}


	/**
	 * Retrieves last sync date/time. If the sync has never run before it returns a very old init date/time
	 *
	 * @param      $source_id
	 * @param bool $reset
	 *
	 * @return string
	 *
	 *
	 * @since version
	 */
	public function getCxIsSyncedToThisDateTime( $source_id, $reset = false )
	{
		$lastSyncDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . 'syncstatus';
		$sync_timestamp_file = $lastSyncDirPath . DIRECTORY_SEPARATOR . "last_sync_jobs_SRCID{$source_id}.txt";

		// if (!file_exists($lastSyncDirPath)) mkdir($lastSyncDirPath); // moved to Environment_Installation->setDirectories()

		if (!file_exists($sync_timestamp_file) || (file_exists($sync_timestamp_file) && filesize($sync_timestamp_file) == 0) || $reset) {
			$handle = fopen( $sync_timestamp_file, "w" );
			fwrite( $handle, $this->interfacer::INIT_SYNC_STARTDATE /*self::INIT_SYNC_STARTDATE*/ );
			fclose( $handle );
			return $this->interfacer::INIT_SYNC_STARTDATE;
		}

		$handle = fopen($sync_timestamp_file, "r") or die("Unable to open file!");
		$lastCronTime = fread($handle,filesize($sync_timestamp_file));
		fclose($handle);

		return $lastCronTime;
	}

}