<?php /** @noinspection PhpUndefinedVariableInspection */
?>

<form method="post" class="undoable">
    <?php wp_nonce_field('settings', '_carerix_nonce_field'); ?>

    <br>
    <h1>General settings</h1>

    <table class="form-table">
		<tbody>
            <?php if($credentials_changed && !$app_token_error): ?>
            <tr>
                <th scope="col" colspan="3">
                    <div class="carerix_msgBox carerix_note">
                        Your settings have been changed. The <b>Synchronize new items from Carerix</b> will start now.
                    </div>
                </th>
            </tr>
            <?php endif; ?>
            
			<?php if(isset($page_error) && $page_error): ?>
			<tr>
				<th scope="col" colspan="3">
					<div class="carerix_msgBox carerix_error">
					Error: you can not proceed. There are errors on this page.  Please correct these errors and try again.
					</div>
				</th>
			</tr>
			<?php endif; ?>

			<?php if(isset($api_error) && $api_error): ?>
			<tr>
				<th scope="col" colspan="3">
					<div class="carerix_msgBox carerix_error">
						Error: can not connect to your Carerix application. The Application Token or the Application Name is not correct.
					</div>
				</th>
			</tr>
			<?php endif; ?>

			<?php if(isset($success) && $success): ?>
			<tr>
				<th scope="col" colspan="3">
					<div class="carerix_msgBox carerix_ok">
					All settings were succesfully saved.
					</div>
				</th>
			</tr>
			<?php endif; ?>

            <tr valign="top">
                <th scope="col" colspan="3">
                    <div class="carerix_msgBox carerix_note">
                        Fill in the application name add the application Token to connect with the Carerix System and click Save. Follow the steps on the wiki page to generate your token: <a href="http://help.carerix.com/en/articles/1958063-wordpress-plugin-activate-api" target="_blank">Generate Token</a>
                        <br/>
                        <br/>
                        <b>IMPORTANT:</b><br/>
                        <ul style="list-style: inherit; padding-left: 16px; margin-top:0;">
                            <li style="margin-bottom: 0;">Don't run this plugin on NGINX. Currenty only Apache is supported</li>
                            <li style="margin-bottom: 0;">Make sure you have enough server memory (e.g. <?php echo CARERIX_MEMORY_LIMIT_MIN; ?>MB). Else the jobs-synchronization process and application forms (for candidates) might fail</li>
                            <li style="margin-bottom: 0;">This plugin was tested to work in different browsers and WordPress versions. However we cannot guarantee, that the plugin will function with all themes, third party plugins and server environments. See <a href="http://development.wiki.carerix.com/cxwiki/doku.php?id=cxwordpress2#known_issues" target="_blank">Plugins & Themes that we black- and whitelisted</a>. (Login credentials are provided earlier to you by Carerix)</li>
                        </ul>
                    </div>
                </th>
            </tr>
        </tbody>
    </table>

    <ul>
        <li class="itemDetail">
            <h2 class="itemTitle">Carerix Application connection</h2>
            <table class="form-table">
                <tbody>
                <tr valign="top">
                    <th scope="row">
                        <label for="app_name">Carerix Application Name</label>
                    </th>
                    <td>
                        <input name="app_name" type="text" id="app_name" value="<?php echo $app_name ?? ""; ?>" class="regular-text <?php if(isset($app_name_error)) echo "carerix_field_error"; ?>">
				        <?php if(isset($app_name_error) && $app_name_error): ?>
                            <p class="carerix_field_error">This field is mandatory</p>
				        <?php endif; ?>
                    </td>
                    <td>
                        <span style="font-style: italic;">If your Carerix system is accessible via https://<b>jdoerecruitment</b>.carerix.net, the application name is <b>jdoerecruitment</b> </span>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">
                        <label for="app_token">Application Token</label>
                    </th>
                    <td>
                        <input name="app_token" type="text" id="app_token" value="<?php echo $app_token ?? ""; ?>" class="regular-text <?php if(isset($app_name_error)) echo "carerix_field_error"; ?>">
				        <?php if(isset($app_token_error) && $app_token_error): ?>
                            <p class="carerix_field_error">This field is mandatory</p>
				        <?php endif; ?>
                    </td>
                    <td>
                        <span style="font-style: italic;">Follow the steps on the wiki page to generate your token: <a href="http://help.carerix.com/en/articles/1958063-wordpress-plugin-activate-api" target="_blank">Generate Token</a></span>
                    </td>
                </tr>
                <?php
                    $css1 = "";
                    $css2 = "";
                    if ( Environment_Settings::getRestApiSync() !== "on" ) {
                        $css1 = 'border: 3px solid red;';
                        $css2 = 'padding-left: 3px;';
                    }
                ?>
                <tr valign="top" style="<?php echo $css1; ?>">
                    <th scope="row" style="<?php echo $css2; ?>">
                        <label for="restapisync_enabled">Synchronize job posts by REST-API</label>
                    </th>
                    <td valign="left" colspan="2">
                        <label>
                            <input disabled type="checkbox" name="restapisync_enabled" id="restapisync_enabled" <?php echo $checked = (isset($restapisync_enabled) && $restapisync_enabled == 'on') ? 'checked' : '';  ?> >
                            <span>As of Dec 1, 2021, synchronizing jobs and other data will be handled through the REST API (instead of the CxTools feed).<br/></span>
                        </label>
                    </td>
                </tr>
                </tbody>
            </table>
        </li>
    </ul>

    <ul>
        <li class="itemDetail">
            <h2 class="itemTitle">Category settings of job posts</h2>
            <table class="form-table">
                <tbody>
                <tr>
                    <th scope='row'>
                        <label for='sync_clean'>Clean categories at sync</label>
                    </th>
                    <td valign='left' colspan='2'>
                        <label>
                            <input type='checkbox' name='sync_clean' id='sync_clean' <?php echo $checked = (isset($sync_clean) && $sync_clean == 'on') ? 'checked' : ''; ?> />
                            Remove empty categories (without vacancies or subcategories) when items are synchronized
                        </label>
                    </td>
                </tr>
                <tr>
                    <?php
                        $interfacer = WP_Interfacer::getInstance();
                        $language = $interfacer->language_major_backend_user();
                        $labels = Utils_Labels::get($language);
                        $publications_label = strtolower($labels['general.header.publications']);
                        $f0_label = strtolower($labels['CxPortal.message.field.singleFunctionGroup']);
                        $f1_label = strtolower($labels['CxPortal.message.field.function']);
                    ?>
                    <th scope='row'>
                        <label for='skip_main_category'>Skip main category</label>
                    </th>
                    <td valign='left' colspan='2'>
                        <label>
                            <input type='checkbox' name='skip_main_category' id='skip_main_category' <?php echo $checked = (isset($skip_main_category) && $skip_main_category == 'on') ? 'checked' : ''; ?> />
                            Yes, skip the main category.<br>
                            <br>
                        </label>
                        The base category <i><?php echo $publications_label; ?></i> holding all vacancies, will be removed. All subcategories (like <i><?php echo $f0_label; ?></i> and <i><?php echo $f1_label; ?></i>) will move 1 step up the hierarchical ladder.<br>
                        Note: be aware that the vacancy post URL structure will change.
                    </td>
                </tr>
                <tr>
                    <th scope='row'>
                        <label for='append_language_suffix'>Append languagecode suffix<br>(for WPML multilingual sites)</label>
                    </th>
                    <td valign='left' colspan='2'>
                        <label>
                            <input type='checkbox' name='append_language_suffix' id='append_language_suffix' <?php echo $checked = (isset($append_language_suffix) && $append_language_suffix == 'on') ? 'checked' : ''; ?> />
                            Yes, append the <i>languagecode suffix</i> like <b>-nl</b>, <b>-fr</b>, <b>-de</b> to category slugs of non default language(s)<br>
                            <br>
                        </label>
                        Note: be aware that the vacancy post URL structure will change.
                    </td>
                </tr>

                </tbody>
            </table>
        </li>
    </ul>

    <ul>
        <li class="itemDetail">
            <h2 class="itemTitle">SEO</h2>
            <table class="form-table">
                <tbody>
                <tr>
                    <th scope='row'>
                        <label for='seo_skip_yoast_meta'>Skip Yoast SEO meta description</label>
                    </th>
                    <td valign='left' colspan='2'>
                        <label>
                            <input type='checkbox' name='seo_skip_yoast_meta' id='seo_skip_yoast_meta' <?php echo $checked = (isset($seo_skip_yoast_meta) && $seo_skip_yoast_meta == 'on') ? 'checked' : ''; ?> />
                            Don't set/overwrite the &lt;meta name="description"&gt; tag for job post pages (while synchronizing).<br>This setting is only operational if the plugin Yoast SEO is installed and active.
                        </label>
                    </td>
                </tr>
                </tbody>
            </table>
        </li>
    </ul>

    <ul>
        <li class="itemDetail">
            <h2 class="itemTitle">Logging and debugging</h2>
            <table class="form-table">
                <tbody>
                <tr valign="top">
                    <th scope="row">
                        <label for="app_reporting">Allow the plugin to report its location</label>
                    </th>
                    <td valign="left" colspan="2">
                        <label>
                            <input type="checkbox" name="app_reporting" id="app_reporting" <?php echo $checked = (isset($app_reporting) && $app_reporting == 'on') ? 'checked' : '';  ?> >
                            By reporting its location, we will be able to track if there are any issuess. Therefor the quality of plugin development will be much better.
                        </label>
                    </td>
                </tr>
                <tr>
                    <th scope='row'>
                        <label for='enable_logging'>Enable developers logging</label>
                    </th>
                    <td valign='left' colspan='2'>
                        <label>
					        <?php $logDirPath = str_replace(get_home_path(), "{WP_ROOT}/", Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::LOG_DIR_CONTAINER) . DIRECTORY_SEPARATOR; ?>
                            <input type='checkbox' name='dev_logging' id='dev_logging' <?php echo $checked = (isset($dev_logging) && $dev_logging == 'on') ? 'checked' : ''; ?> />
                            Log-events will be written to: <?php echo $logDirPath; ?>
                        </label>
                    </td>
                </tr>
                </tbody>
            </table>
        </li>
    </ul>

    <p class="submit">
        <input type="submit" name="submit" id="submit" class="button button-primary" value="Save changes">
    </p>

    <p>&nbsp;</p>

    <hr>

    <h1>Manual synchronization</h1>

    <ul id="section_sync">
        <li>
            <table class="form-table">
                <tbody>
                <tr valign="top">
                    <th scope="col" colspan="3">
                        <div class="carerix_msgBox carerix_note">
                            Have you made changes in your Carerix system and do you want them to be updated now on this website? Click on <i style="background:#abb3bf; padding: 3px; line-height: 2em;">Synchronize new/changed items from Carerix</i> to immediately synchronize with your system. Otherwise they will be updated during the automatic synchronization which takes place every 10 minutes.<br/>
	                        <?php if (Environment_Settings::getRestApiSync() == "on") : ?>
                                If you want to make sure all eligible items are synchronized, you can do a full-sync by using <i style="background:#abb3bf; padding: 3px; line-height: 2em;">Enforce full sync of all items from Carerix</i>. In general you don't need to use this button.
	                        <?php endif; ?>
                        </div>

                        <div id="sync_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process finished with success
                        </div>

                        <div id="sync_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>
                    </th>
                </tr>
                <tr>
                    <td colspan="2">
				        <?php
				        $disabled = '';
				        if(isset($error) && $error):
					        $disabled = 'disabled';
				        endif;
				        ?>

                        <button <?php echo $disabled; ?> id="app_cron" name="sync_now" class="button button-primary <?php echo $disabled; ?>"><i class="fa fa-refresh fa-spin wait" style="display: none;"></i> Synchronize new/changed items from Carerix </button>
                        <?php if (Environment_Settings::getRestApiSync() == "on") : ?>
                            <span style="margin-left: 20px;"></span>
                            <button <?php echo $disabled; ?> id="app_cron_full" name="sync_now_full" class="button button-primary <?php echo $disabled; ?>"><i class="fa fa-refresh fa-spin wait" style="display: none;"></i> Enforce full sync of all items from Carerix </button>
                        <?php endif; ?>
                    </td>
                </tr>
                <tr valign="top">
                    <td colspan="2">
		                <?php
		                $link = get_bloginfo('url') . '/?carerix_sync';
		                ?>
                        Or you us the following link to sync new/changed items with verbose logging output:<br/>
                        <a href='<?php echo $link; ?>' target='_blank'><?php echo $link; ?></a>
                    </td>
                </tr>
                <tr valign="top">
                    <th colspan="3">
                        <div id="jobs_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for Jobs sources has started <span class="progress"></span>
                        </div>

                        <div id="jobs_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for Jobs sources finished with success
                        </div>

                        <div id="jobs_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for Jobs sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>

                        <div id="promotionals_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for Promotional Jobs sources has started
                        </div>

                        <div id="promotionals_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for Promotional Jobs sources finished with success
                        </div>

                        <div id="promotionals_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for Promotional Jobs sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>

                        <div id="candidates_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for Candidate sources has started
                        </div>

                        <div id="candidates_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for Candidates sources finished with success
                        </div>

                        <div id="candidates_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for Candidates sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>

                        <div id="people_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for People sources has started
                        </div>

                        <div id="people_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for People sources finished with success
                        </div>

                        <div id="people_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for People sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>

                        <div id="companies_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for Companies sources has started
                        </div>

                        <div id="companies_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for Companies sources finished with success
                        </div>

                        <div id="companies_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for Companies sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>

                        <div id="newsletters_all_progress" class="carerix_msgBox carerix_note" style="display: none;">
                            <i class="fa fa-refresh fa-spin wait"></i> The synchronization process for Newsletters sources has started
                        </div>

                        <div id="newsletters_all_success" class="carerix_msgBox carerix_ok" style="display: none;">
                            The synchronization process for Newsletters sources finished with success
                        </div>

                        <div id="newsletters_all_error" class="carerix_msgBox carerix_error" style="display: none;">
                            Error: the synchronization process for Newsletters sources failed. Please try again. If the problem persists please contact your Carerix administrator.
                        </div>
                    </th>
                </tr>
                </tbody>
            </table>
        </li>

    </ul>


    <p>&nbsp;</p>

    <hr>

    <h1>Reset Plugin</h1>

    <ul>
        <li>
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th colspan="3">
                            <div class="carerix_msgBox carerix_note">
                                The reset button will reset your plugin to the default installation, except the App Name and App Token.<br>
                                Every option, source and form will be deleted, leaving only the default ones.<br>
                                Every post and page created automatically by the plugin will be deleted and a resynchronization will be done for the default source.<br>
                            </div>
                        </th>
                    </tr>
                    <tr valign="top">
                        <th>
                            <button id="reset" name="reset" class="button button-primary"><i class="fa fa-refresh fa-spin wait" style="display: none;"></i><span class='text'>Reset Plugin to defaults</span></button>
                        </th>
                    </tr>
                </tbody>
            </table>
        </li>
    </ul>
</form>

<?php
$ajax_nonce_sync_all  = wp_create_nonce( "sync_all" );
$ajax_nonce_reset_all = wp_create_nonce( "reset_all" );
?>
<script>
    jQuery(document).ready(function(e){
        var $ = jQuery;
        
        $("#app_cron").click(function(e){
            $(this).attr('disabled','disabled');
           $("#sync_all_error").hide();
           $("#sync_all_success").hide();
           e.preventDefault(); 
           var spinner = $(this).find('.wait').show();
           
           <?php /* old: var source_types = ['jobs', 'promotionals', 'candidates', 'people', 'companies', 'newsletters']; */ ?>
           var source_types = ['jobs'];
           var finished = {};
           
           var setFinished = function(key) {
                   finished[key] = true;
           };
           
           getAllFinished = function() { return finished; };
           
           var errorCount = {};
           var setErrorCount = function(type, step) {
               if(errorCount[type]) {
                   errorCount[type][step] = errorCount[type][step] + 1;
               } else {
                   errorCount[type] = {};
                   errorCount[type][step] = 1;
               }
           };
           
           var getErrorCount = function() { return errorCount; }
           var hasMaximumErrors = function(type, step) {
               var maxErrors = 3;
                if(errorCount[type] && errorCount[type][step] && errorCount[type][step] == maxErrors) {
                   return true;
               }
               return false;
           };
           
           var checkStatusCron = function(){
                var allFinished = true;
                var finished = getAllFinished();
                
                for(var j=0; j < source_types.length; j++) {
                    if(!finished[source_types[j]]) allFinished = false;
                }
                if(allFinished) {
                    spinner.hide();
                    $(self).removeAttr('disabled');
                    
                    $('#reset').removeAttr('disabled');
                    $('#reset').find('.wait').hide();
                    // $('#reset').find('.text').html('Reset to default');
                }
           };
           
           var self = this;
           
           var cronFunction = function(source_type, timeout) {
                var progress_selector = "#"+source_type+"_all_progress";
                var error_selector = "#"+source_type+"_all_error";
                var success_selector = "#"+source_type+"_all_success";
           
               $(error_selector).hide();
                $(success_selector).hide();
                $(progress_selector).show();
           
           
               setTimeout(function() {
                    
                    
               
                   var cron = function(source_type, pass){

                            var data = {
                                action: 'sync_all',
                                security: '<?php echo $ajax_nonce_sync_all; ?>',
                                source_type: source_type,
                                page: pass
                            };
                            var maxError = hasMaximumErrors(source_type, pass);
                            if(maxError)  {
                                $(progress_selector).fadeOut();
                                $(progress_selector + " span.progress").html("");
                                $(success_selector).fadeOut();
                                $(error_selector).fadeIn();
                                setFinished(source_type);
                                return;
                            }

                            $.post(ajaxurl, data, function(response, status, jqXHR){
                                
                                if(status != 'success' || response == null) {
                                    setErrorCount(source_type, pass);
                                    cron(source_type, pass);
                                    
                                } else {
                                    if(response.sync_success == true) {
                                        if(response.has_more == true) {
                                            $(progress_selector + " span.progress").html("(processed batch #" + (response.page+1) + ")");
                                            cron(source_type, pass+1);
                                        } else {
                                            $(success_selector).fadeIn();
                                            $(error_selector).fadeOut();
                                            $(progress_selector).fadeOut();
                                            $(progress_selector + " span.progress").html("");
                                            setFinished(source_type);
                                        }
                                    } else {
                                        setErrorCount(source_type, pass);
                                        cron(source_type, pass);
                                    }
                                }
                                checkStatusCron();
                                
                            }, 'json').fail(function(jqXHR, textStatus, errorThrown){ 
                                setErrorCount(source_type, pass);
                                cron(source_type, pass);
                                checkStatusCron();
                            });
                       };  
                       cron(source_type, 0);
               }, timeout);
           };
           
           for(var i= 0; i < source_types.length; i++) {
               cronFunction(source_types[i], i * 1000);
           }
           return false;
        });
        
        $("#app_cron_full").click(function(e){
           $(this).attr('disabled','disabled');
           $("#sync_all_error").hide();
           $("#sync_all_success").hide();
           e.preventDefault();
           var spinner = $(this).find('.wait').show();

           <?php /* old: var source_types = ['jobs', 'promotionals', 'candidates', 'people', 'companies', 'newsletters']; */ ?>
           var source_types = ['jobs'];
           var finished = {};

           var setFinished = function(key) {
                   finished[key] = true;
           };

           getAllFinished = function() { return finished; };

           var errorCount = {};
           var setErrorCount = function(type, step) {
               if(errorCount[type]) {
                   errorCount[type][step] = errorCount[type][step] + 1;
               } else {
                   errorCount[type] = {};
                   errorCount[type][step] = 1;
               }
           };

           var getErrorCount = function() { return errorCount; }
           var hasMaximumErrors = function(type, step) {
               var maxErrors = 3;
                if(errorCount[type] && errorCount[type][step] && errorCount[type][step] == maxErrors) {
                   return true;
               }
               return false;
           };

           var checkStatusCron = function(){
                var allFinished = true;
                var finished = getAllFinished();

                for(var j=0; j < source_types.length; j++) {
                    if(!finished[source_types[j]]) allFinished = false;
                }
                if(allFinished) {
                    spinner.hide();
                    $(self).removeAttr('disabled');

                    $('#reset').removeAttr('disabled');
                    $('#reset').find('.wait').hide();
                    // $('#reset').find('.text').html('Reset to default');
                }
           };

           var self = this;

           var cronFunction = function(source_type, timeout) {
                var progress_selector = "#"+source_type+"_all_progress";
                var error_selector = "#"+source_type+"_all_error";
                var success_selector = "#"+source_type+"_all_success";

               $(error_selector).hide();
               $(success_selector).hide();
               $(progress_selector).show();


               setTimeout(function() {



                   var cron = function(source_type, pass){

                            var data = {
                                action: 'sync_all_full',
                                security: '<?php echo $ajax_nonce_sync_all; ?>',
                                source_type: source_type,
                                page: pass
                            };
                            var maxError = hasMaximumErrors(source_type, pass);
                            if(maxError)  {
                                $(progress_selector).fadeOut();
                                $(progress_selector + " span.progress").html("");
                                $(success_selector).fadeOut();
                                $(error_selector).fadeIn();
                                setFinished(source_type);
                                return;
                            }

                            $.post(ajaxurl, data, function(response, status, jqXHR){

                                if(status != 'success' || response == null) {
                                    setErrorCount(source_type, pass);
                                    cron(source_type, pass);

                                } else {
                                    if(response.sync_success == true) {
                                        if(response.has_more == true) {
                                            $(progress_selector + " span.progress").html("(processed batch #" + (response.page+1) + ")");
                                            cron(source_type, pass+1);
                                        } else {
                                            $(success_selector).fadeIn();
                                            $(error_selector).fadeOut();
                                            $(progress_selector).fadeOut();
                                            $(progress_selector + " span.progress").html("");
                                            setFinished(source_type);
                                        }
                                    } else {
                                        setErrorCount(source_type, pass);
                                        cron(source_type, pass);
                                    }
                                }
                                checkStatusCron();

                            }, 'json').fail(function(jqXHR, textStatus, errorThrown){
                                setErrorCount(source_type, pass);
                                cron(source_type, pass);
                                checkStatusCron();
                            });
                       };
                       cron(source_type, 0);
               }, timeout);
           };

           for(var i= 0; i < source_types.length; i++) {
               cronFunction(source_types[i], i * 1000);
           }
           return false;
        });
        
        $("#reset").click(function(e){
            e.preventDefault;
            
            var self = this;
            $(self).attr('disabled','disabled');
            var spinner = $(this).find('.wait').show();
            var textContainer = $(this).find('.text');
            var data = {
               action: 'reset_all',
               security: '<?php echo $ajax_nonce_reset_all; ?>',
            };
           textContainer.html('Resetting to default. Please wait.. ');
            
            var resetCron = function() {
                $.post(ajaxurl, data, function(response, status, jqXHR){
               
                    if(response && response.ok) {
                         spinner.hide();
                         $(self).removeAttr('disabled');
                         $(textContainer).html('Reset process was successful. Starting the synchronize process.. ');
                         $("#app_cron_full").click();
                    } else {
                        $(textContainer).html('Reset process didn\'t complete. Retrying. Please wait .. ');
                        resetCron();
                         
                    }
               
                }, 'json').fail(function(){
                    $(textContainer).html('Reset process didn\'t complete. Retrying. Please wait .. ');
                    resetCron();
                });
            };
            
           resetCron();
           return false;
        });
        

       <?php if($credentials_changed && !$app_token_error): ?>
        
        $("#app_cron_full").click();
        
        <?php endif; ?>
    });
</script>