<?php

namespace Carerix\CxWordpress;

use Carerix;


class Applicationsubmissionqueue {
	const SUBMISSION_QUEUE_DIR = "applicationsubmissionqueue";

	protected string $filename = "";
	protected array $form_data;
	protected ?int $publication_id = null;

	/**
	 * Generates a new hashkey based on emailadress or PHP session ID (as a fallback)
	 * If a form-application is loaded from the queue, the hashkey is extracted from the filename
	 **
	 * @return string
	 */
	public function getStoredHashKey() : string {
		if ( strlen($this->filename) ) {
			// E.g. My_cv.pdf_CVTag_d72c6e567a68468ce74142f39459a943 -> d72c6e567a68468ce74142f39459a943
			$items   = explode("_", $this->filename);
			$hashkey = end($items) ?? "";
		} else {
			$hashkey = ""; // don't have one
		}

		return $hashkey;
	}

	public function queueAsPendingApplicationSubmission( string $hashkey, array $input_data, int $publication_id = null ) : bool {
		$filename = $hashkey; // $this->getStoredHashKey($input_data, $publication_id);
		$filepath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . self::SUBMISSION_QUEUE_DIR . DIRECTORY_SEPARATOR . $filename;

		$bin_data = [$input_data, $publication_id];
		$json_data = json_encode($bin_data, JSON_OBJECT_AS_ARRAY);

		$handle = fopen( $filepath, "w" );
		fwrite( $handle, $json_data );
		fclose( $handle );

		$this->filename = $filename;
		return true;
	}

	/**
	 * Loads a pending application form which is stored by queueAsPendingApplicationSubmission() or in the queue
	 *
	 * @param bool $fromQueue   false=load from earlier queueAsPendingApplicationSubmission(), true=load one from queue (disk storage)
	 *
	 * @return bool
	 */
	public function loadPendingApplicationSubmission(bool $fromQueue = false ) : bool {
		if ( $fromQueue ) {
			$dirpath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . self::SUBMISSION_QUEUE_DIR . DIRECTORY_SEPARATOR;
			$this->filename = scandir($dirpath)[2] ?? "NON-EXISTING-FILE";
		}


		$filepath = $this->getFilePathForFilename( $this->filename );


		if (!file_exists($filepath)) {
			$this->form_data = [];
			$this->publication_id = null;
			return false;
		}

		$handle = fopen($filepath, "r");
		$json_data = fread($handle, filesize($filepath));
		fclose($handle);

		$bin_data = json_decode($json_data, true);
		$this->form_data = $bin_data[0];
		$this->publication_id = $bin_data[1];

		return true;
	}

	public function deleteProcessedApplicationSubmission() : bool {
		$filepath = $this->getFilePathForFilename( $this->filename );

		if (!file_exists($filepath)) {
			return false;
		}

		$this->form_data = [];
		$this->publication_id = null;
		$this->filename = "";

		unlink($filepath);

		return true;
	}

	public function getFormData() : array {
		return $this->form_data;
	}

	public function getPublicationId() : int {
		return $this->publication_id;
	}

	public function getFilename(): string {
		return $this->filename;
	}

	private function getFilePathForFilename( string $filename) : string {
		$filepath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . 'applicationsubmissionqueue' . DIRECTORY_SEPARATOR . $filename;
		return $filepath;
	}
}